/*
 * Copyright 2023 Alexey Lyashko
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the “Software”), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished
 * to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef TRIMP_TRIMP_H
#define TRIMP_TRIMP_H

#include "activation_type.h"

#if defined(_MSC_BUILD) && defined(_WINDOWS)
#define DLL_EXPORT __declspec(dllexport)
#else
#define DLL_EXPORT
#endif


#if defined(cplusplus)
extern "C" {
#endif

/*
 * Net type.
 * This denotes whether the net should be [internally] treated as recurrent or a regular.
 */
enum net_type
{
    MLP,        // The net is a regular Multilayer Perceptron.
    Elman       // The net is an implementation of the Elman recurrent net.
};

typedef struct net_t*  pnet_t;

/*
 * Allocates new multilayer perceptron
 * numLayers - Number of layers in the net (including the input and output layers).
 * neuronsPerLayer - Pointer to an array of int holding number of neurons for each layer.
 * neuronActivationType - Pointer to an array of enum activation_type values for each layer.
 * return - Pointer to the newly allocated net or NULL.
 */
DLL_EXPORT pnet_t Net_new(int numLayers, int *neuronsPerLayer, enum activation_type *neuronActivationType);

/*
 * Allocates new multilayer perceptron with the ability to set different activation types for each neuron (even
 * withing a single layer).
 * numLayers - Number of layers in the net (including the input and output layers).
 * neuronsPerLayer - Pointer to an array of int holding number of neurons for each layer.
 * neuronActivationType - Pointer to array of pointers to arrays of activation type values for each neuron in each layer.
 * @return - Pointer to the newly allocated net or NULL.
 */
DLL_EXPORT pnet_t Net_new_ex(int numLayers, int *neuronsPerLayer, enum activation_type **neuronActivationType);

/*
 * Releases all the resources allocated for the net and destroys the net itself.
 * net - Pointer to a net to be freed.
 */
DLL_EXPORT void Net_free(pnet_t net);

/*
 * Runs a single iteration of the network training cycle.
 * net Pointer to the net.
 * inputData - Array of input values. The size of the array MUST be equal to the amount of input neurons.
 * outputData - Array that receives the output values. The size of the array MUST be equal to the amount of output neurons.
 * expectedResult - Array of values containing the expected results. The size of the array MUST be equal to the amount of output neurons.
 * return - Net's error value.
 */
DLL_EXPORT double NetTrain(pnet_t net, double *inputData, double *outputData, double *expectedResult);

/*
 * Runs a single iteration of data processing.
 * net - Pointer to the net.
 * inputData - Array of input values.
 * outputData - Array that receives the output values.
 */
DLL_EXPORT void NetExec(pnet_t net, double *inputData, double *outputData);

/*
 * Sets the type of the net
 * net - Pointer to the net.
 * type - Type of the network (MLP or Elman).
 */
DLL_EXPORT void NetSetType(pnet_t net, enum net_type type);

/*
 * Sets the learning rate for the net.
 * net - Pointer to the net.
 * learningRate - Value of the learning rate.
 */
DLL_EXPORT void NetSetRate(pnet_t net, double learningRate);

/*
 * Sets the value of the momentum.
 * net - Pointer to the net.
 * momentum - Value of the momentum.
 */
DLL_EXPORT void NetSetMomentum(pnet_t net, double momentum);

#if defined(cpluspluc)
}
#endif

#endif //TRIMP_TRIMP_H
